package org.erif.util;

/** General credit card constants and information.
 * <table>
 * <tr><th>Card Type</th><th>Prefix</th><th>Length</th><th>Check Digit</th></tr>
 * <tr><td>Mastercard</td><td>51-55</td><td>16</td><td>mod 10</td></tr>
 * <tr><td>Visa</td><td>4</td><td>13, 16</td><td>mod 10</td></tr>
 * <tr><td>AmEx</td><td>34, 37</td><td>15</td><td>mod 10</td></tr>
 * <tr><td>Diners Club/<br />Care Blanche</td><td>300-305, 36, 38</td><td>14</td><td>mod 10</td></tr>
 * <tr><td>Discover</td><td>6011</td><td>16</td><td>mod 10</td></tr>
 * <tr><td>enRoute</td><td>2014, 2149</td><td>15</td><td>any</td></tr>
 * <tr><td>JCB</td><td>3</td><td>16</td><td>mod 10</td></tr>
 * <tr><td>JCB</td><td>2131, 1800</td><td>15</td><td>mod 10</td></tr>
 * </table>
 *
 * TODO: currently only supports MC/VISA/AMEX for typechecking
 *
 */

public class CCUtil {
  public static final int INVALID = -1;
  public static final int MASTERCARD = 0;
  public static final int VISA = 1;
  public static final int AMEX = 2;
  public static final int DINERS = 3;
  public static final int CARTBLANCHE = 4;
  public static final int DISCOVER = 5;
  public static final int ENROUTE = 6;
  public static final int JCB = 7;

  public static final int MINLENGTH = 13;

  public static int getTypeFromNumber(String ccnum) {
    char c;
    if (ccnum == null || ccnum.length() < MINLENGTH) return INVALID;
    switch (ccnum.charAt(0)) {
      case '5': //mastercard: 16 digits, a 5 followed by a digit between 1 and 5
        if (ccnum.length() != 16 || 
            (c = ccnum.charAt(1)) < '1' || 
            c > '5') return INVALID;
        return MASTERCARD;
      case '4': //visa: 13 or 16 digits, starting with a 4
        if (ccnum.length() != 13 && ccnum.length() != 16) return INVALID;
        return VISA;
      case '3': //amex: 15 digits, a 3 followed by either a 4 or 7
        if (ccnum.length() != 15 ||
            ((c = ccnum.charAt(1)) != '4' && c != '7')) return INVALID;
        return AMEX;
      default:
        //discover: starts with 6011, 16 digits long
        //diner: starts with 300-305, 36, or 38; 14 digits long
        return INVALID;
    }
  }

  public static int mod10(String ccnum) {
    int sum = 0, digit;
    boolean odd = false;
    if (ccnum.length() < MINLENGTH) return -1;
    for (int i = ccnum.length() - 1 ;i>=0;i--,odd=!odd) {
      digit = ((int)ccnum.charAt(i))-'0';
      if (digit < 0 || digit > 9) return -1;
      if (odd) { digit *= 2; if (digit > 9) digit = (digit % 10) + 1; }
      sum += digit;
    }
    return sum % 10;
  }

  public static boolean verify(String ccnum) {
    if (mod10(ccnum) != 0) return false;
    if (getTypeFromNumber(ccnum) < 0) return false;
    return true;
  }

  /** validates based on card type and mod10 rule; please use one of the appropriate class constants to specify the credit card type. */
  public static boolean verify(String ccnum, int type) {
    if (verify(ccnum)) return type == getTypeFromNumber(ccnum);
    else return false;
  }

  public static void main(String[] args) {
    System.err.println("mod10: "+mod10(args[0]));
    System.err.println("type: "+getTypeFromNumber(args[0]));
    System.err.println("verify: "+verify(args[0]));
  }

}
